package Lingua::Stem::Snowball;

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
@EXPORT = qw(SnowballStemmers snowball);

$VERSION = '0.02';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
        if ($! =~ /Invalid/) {
            $AutoLoader::AUTOLOAD = $AUTOLOAD;
            goto &AutoLoader::AUTOLOAD;
        }
        else {
                croak "Your vendor has not defined Lingua::Stem::Snowball macro $constname";
        }
    }
    no strict 'refs';
    *$AUTOLOAD = sub () { $val };
    goto &$AUTOLOAD;
}

sub new {
        my ($class, %opt)=@_;
        $class = ref($class) || $class;
        my $self={};

	die "No lang option" if ( ! length $opt{lang} ); 
	$self->{LANG} = $opt{lang}; 
	$self->{LANG_ID} = Lingua::Stem::Snowball::get_stemmer_id( $opt{lang} );

	if (  $self->{LANG_ID} < 0 ) {
		if ( $self->{LANG_ID} == -1 ) {
			die "There is no lang '$opt{lang}'";
		} elsif ( $self->{LANG_ID} == -2 ) {
			die "Can't call init for lang '$opt{lang}'";
		}
		die "Unknown error";
	}

        bless ($self, $class);

        return $self;
}

sub stem {
	my ($self,$word, $rr) = @_;

	return undef if ! length $word;

	my $lexem;
	my $res = Lingua::Stem::Snowball::do_stem($self->{LANG_ID}, $word, $lexem );
	if ( $res < 0 ) {
		die "Error in Lingua::Stem::Snowball::do_stem";
	}
	$$rr = $res if ( ref $rr );

	return $lexem; 
}

sub SnowballStemmers {
	my @lang;
	Lingua::Stem::Snowball::get_stemmer_list( \@lang );
	return @lang;
}

sub snowball {
	my ($lang, $word, $rr) = @_;

	return Lingua::Stem::Snowball->new( lang=>$lang )->stem( $word, $rr );	
}

bootstrap Lingua::Stem::Snowball $VERSION;

1;

__END__

=head1 NAME

Lingua::Stem::Snowball - Perl interface to Snowball stemmers. 

=head1 PREREQUISITIES

Snowball - http://snowball.sourceforge.net/

=head1 SYNOPSIS

use  Lingua::Stem::Snowball;

OO interface:

my $lang = 'english';
my $dict=Lingua::Stem::Snowball->new( lang=>$lang );

my $lemm = $dict->stem( $word );

my $lemm = $dict->stem( $word, \$is_stemmed );

Plain interface:

my @lang = SnowballStemmers();

my $lemm = snowball( $lang, $word );

my $lemm = snowball( $lang, $word, \$is_stemmed );

=head1 DESCRIPTION

See README.

=head1 AUTHOR

Teodor Sigaev,  teodor@stack.net

=head1 SEE ALSO

http://snowball.sourceforge.net/

=cut


